// Programme qui affiche l'ensemble de Mandelbrot.
// Auteur : Jrme Conus

#include <QApplication>
#include <QImage>
#include <QLabel>

#include <cmath>

#include "colorgradient.h"

// Constantes
const int IMAGE_WIDTH = 1024;
const int IMAGE_HEIGHT = 1024;

const int MAX_ITERATION = 256;

const unsigned int WHITE = 0x0FFFFFFFF;
const unsigned int BLACK = 0x0FF000000;

// Dclaration des fonctions
bool isInMandelbrotSet(double Ca, double Cb, double* pColorPosition = 0);

// Programme principal
int main(int ArgC, char *ArgV[]) {
    QApplication App(ArgC, ArgV);

    // Construit un dgrad
    ColorGradient MyGradient;
    MyGradient.setKey(0,   QColor("#800000")); // rouge fonc
    MyGradient.setKey(0.2, QColor("#FFFF00")); // jaune
    MyGradient.setKey(0.3, QColor("#FFA500")); // orange
    MyGradient.setKey(0.6, QColor("#A020F0")); // pourpre
    MyGradient.setKey(0.8, QColor("#191970")); // bleu nuit
    MyGradient.setKey(1,   QColor("#800000")); // rouge fonc

    // Construit une image vide
    QImage Image(IMAGE_WIDTH, IMAGE_HEIGHT, QImage::Format_ARGB32);
    Image.fill(WHITE);

    // Dtermine l'intervalle  afficher
    QRectF ComplexArea(-2.2, -1.5, 3, 3);

    // Vrifie si les points font partie de l'ensemble de Mandelbrot
    for (int Line = 0; Line < IMAGE_HEIGHT; ++Line) {
        for (int Col = 0; Col < IMAGE_WIDTH; ++Col) {
            double Ca = Col * ComplexArea.width() / IMAGE_WIDTH + ComplexArea.left();
            double Cb = Line * ComplexArea.height() / IMAGE_HEIGHT + ComplexArea.top();

            double ColorPosition = 0;
            if (isInMandelbrotSet(Ca, Cb, &ColorPosition))
                Image.setPixel(Col, Line, BLACK);
            else
                Image.setPixel(Col, Line, MyGradient.colorAt(ColorPosition/MAX_ITERATION).rgb());
        }
    }

    // Sauvegarde l'image obtenue
    Image.save("mandelbrot.png");

    // Affiche l'image dans une fentre
    QPixmap MandelbrotPixmap = QPixmap::fromImage(Image);
    QLabel Label;
    Label.setWindowTitle("Mandelbrot");
    Label.setPixmap(MandelbrotPixmap);
    Label.setMinimumSize(IMAGE_WIDTH, IMAGE_HEIGHT);
    Label.show();

    return App.exec();
}

// Dtermine si le nombre complexe dfini par Ca+iCb fait
// partie de l'ensemble de MandelBrot.
bool isInMandelbrotSet(double Ca, double Cb, double* pColorPosition) {
    double Za = 0;
    double Zb = 0;

    int Iteration = 0;

    while (Za*Za + Zb*Zb < 4. && Iteration < MAX_ITERATION) {
        double ZaTemp = Za * Za - Zb * Zb + Ca;
        Zb = 2. * Za * Zb + Cb;
        Za = ZaTemp;
        ++Iteration;
    }

    if (pColorPosition) {
        // Calcul de la position de la couleur selon l'algo 'normalized iteration count'
       *pColorPosition = Iteration + (-0.5 - log10(log10(Za*Za+Zb*Zb)*0.5))/0.3;
    }

    return Iteration == MAX_ITERATION;
}
